/*
 * Player.java
 *
 * Created on 26. november 2004, 10:10
 */

package checkers;

/**
 * The design to implement the player was inspired from the <U>Object-Oriented Software Development Using java</U> book in chapter 11.3. Two player objects implements an abstract method play() from the Player class. Both of them extend the Thread Java Foundation Class. Consequently, the player that has not the turn wait() until he is notify() by it opponent. This passing the baton technique prevents both threads to access the critical section (the board) at the same time. Mutual exclusion is then ensured. When a player object is constructed, his name and his colour are passed in argument to the constructor.
 * @author k380h11
 */
abstract public class Player extends Thread{
    
    Player turn;
    protected Player next;
    protected String id;
    protected boolean color;
    
    /**
     * Creates a new instance of Player
     * @param id name of the player
     * @param color color of the player
     */
    public Player(String id, boolean color) {
        super();
        this.id = id;
        this.color = color;
        start();
    }
	/**
         * This method is an abstract form with is used by Jess and Human
         * to tell where to move their pieces.
         */    

    abstract public void play();
    
    /**
     * run the new thread
     */    
    public synchronized void run() {
        while (!Checker.getInstance().isOver()) {
            while (turn !=  this) {
                try {
                    wait();
                } catch (InterruptedException e) { return ;}
            }
            Checker.getInstance().displayMessage("Player (" + (color?"white":"red")+ ") "+ id + " `s turn");
            if (Board.getInstance().count(!color) == 0)
                {Checker.getInstance().win(color);}
            else if (Board.getInstance().count(!color) == 0)
                {Checker.getInstance().win(color);}
            play();
            turn = null;
        }
    }
	/**
         * This method is used to synchronized the players so that only one player gets the next turn.
         * @param p next player
         */

    public synchronized void setNext(Player p) {
        next = p;
    }
	/**
         * This method is used to synchronized the hasturn so that only one player can move a piece.
         */
    public synchronized void hasTurn(){
        turn = this;
        Checker.getInstance().turn = color;
        turn.notify();
    }
	/**
         * This method is used to synchronized the getturn so that only get the turn and has the ablity to move a piece.
         * @return the player who's got the turn
         */
    public synchronized Player getTurn(){
        return turn;
    }
    	/**
         * This method is used to convert the player informaation into a string.
         * @return a string containing the information of the player
         */
    public String toString() {
        return turn.id + " " + (turn.color ? "white" : "red");
    }
}
